import es
import gamethread
import time

info = es.AddonInfo()
info.name = 'DM EndRound'
info.version = '2.0'
info.basename = 'dmendround'
info.description = 'Forces "Round Draw" when the round time has elapsed, meant to be used on DeathMatch servers where rounds normally last infinitely'
info.url = 'http://addons.eventscripts.com/addons/view/dmendround/'

# server variable stuff...
version_variable = es.ServerVar('dm_endround_v', info.version, '%s version number'%info.name)
version_variable.set(info.version)
version_variable.makepublic()

dm_realround = es.ServerVar('dm_realround', 0, 'Boolean value telling whether a round is active')
isenabled = es.ServerVar('dm_endround', 1, 'Is %s enabled'%info.name)
dm_endround_effect = es.ServerVar('dm_endround_effect', 1, 'Play sound and display message or not?')
# end of server variable stuff...

mp_roundtime = es.ServerVar('mp_roundtime')
roundtime = float(mp_roundtime)*60.0
last_dm_roundtime = 0.0

laststart = time.time()

ROUND_DRAW_SOUND = 'radio/rounddraw.wav'

def _ticker():
    now = time.time()
    if now-laststart >= roundtime:
        round_draw()

def _start_ticking():
    global isticking
    if not isticking:
        es.addons.registerTickListener(_ticker)
        isticking = True

def _stop_ticking():
    global isticking
    if isticking:
        es.addons.unregisterTickListener(_ticker)
        isticking = False
    
isticking = False
    
def load():
    if not es.exists('command', 'dm_roundtime'):
        es.regcmd('dm_roundtime', '%s/dm_roundtime'%info.basename, 'Overrides mp_roundtime for %s'%info.name)

def unload():
    if isticking:
        es.addons.unregisterTickListener(_ticker)

def dm_roundtime():
    global roundtime, last_dm_roundtime
    argc = es.getargc()
    if argc == 1:
        mins, secs = divmod(last_dm_roundtime, 60)
        es.dbgmsg(0, '"dm_roundtime" = "%d:%02d" min. %0.6f'%(
            mins, secs, 0.0,
            )
                  )
        es.dbgmsg(0, ' - If nonzero, overrides mp_roundtime for the value %s uses for round length'%(
            info.name)
                  )
        es.dbgmsg(0, ' - Can be either integer, floating point number or string of form "min:sec"')
        mins, secs = divmod(roundtime, 60)
        es.dbgmsg(0, ' - Currently effective roundtime = %d:%02d'%(
            mins, secs,
            )
                  )
        return
    elif argc == 2:
        timearg = es.getargv(1)
        try:
            mins = float(timearg)
            secs = 0.0
        except ValueError:
            try:
                mins, secs = (float(x) for x in timearg.split(':', 1))
            except ValueError:
                es.dbgmsg(0, 'Invalid value %s for dm_roundtime'%timearg)
                return
    elif argc == 4:
        if es.getargv(2) == ':':
            try:
                mins = float(es.getargv(1))
                secs = float(es.getargv(3))
            except ValueError:
                es.dbgmsg(0, 'Invalid value %s for dm_roundtime'%es.getargs())
                return
    else:
        es.dbgmsg(0, 'Invalid value %s for dm_roundtime'%es.getargs())
        return
    
    secs = int(secs + mins*60)
    if secs <= 0:
        secs = 0
        roundtime = float(mp_roundtime)*60.0
    else:
        roundtime = float(secs)
    
    last_dm_roundtime = secs

def round_draw():
    laststart = time.time() #just to stop the ticker from doing infinite loop
    if int(isenabled):
        dm_realround.set(0)
        _stop_ticking()
        if int(dm_endround_effect):
            for userid in es.getUseridList():
                es.playsound(userid, ROUND_DRAW_SOUND, 1.0)
                # the following only works in CS:S, but who cares, it is just extra
                es.setplayerprop(userid, 'CCSPlayer.m_iPlayerState', 1)
            es.centermsg('Round Draw!')
            gamethread.delayed(2, invoke_round_draw)
        else:
            invoke_round_draw()

def invoke_round_draw():
    es.ServerCommand('endround')
    
# ALL THE RANDOM EVENTS:
# ROUND_START
def round_start(ev):
    global laststart
    laststart = time.time()
    _start_ticking()

def round_freeze_end(ev):
    round_start(ev)
    dm_realround.set(1)

def game_start(ev):
    round_start(ev)

def dod_round_start(ev):
    round_start(ev)

def dod_round_active(ev):
    round_start(ev)
    dm_realround.set(1)

def teamplay_round_start(ev):
    round_start(ev)

def teamplay_round_active(ev):
    round_start(ev)
    dm_realround.set(1)

# ROUND_END
def round_end(ev):
    dm_realround.set(0)
    _stop_ticking()

def game_end(ev):
    round_end(ev)

def dod_game_over(ev):
    round_end(ev)

def dod_round_win(ev):
    round_end(ev)

def tf_game_over(ev):
    round_end(ev)

def teamplay_round_win(ev):
    round_end(ev)

def teamplay_game_over(ev):
    round_end(ev)

